use rocket::response::status::BadRequest;
use serde_json::json;
use std::str::FromStr;

/// API error response with a JSON payload.
pub type ApiError = BadRequest<serde_json::Value>;

/// Fallible API response.
///
/// This type represents the result of an API operation that can either succeed with a
/// JSON payload (`serde_json::Value`) or fail with an `ApiError`.
pub type FallibleApiResponse = Result<serde_json::Value, ApiError>;

/// A trait to generate a Model API representation in JSON format.
///
/// This trait is intended for data models that need to be serialized
/// into a JSON representation suitable for public APIs.
pub trait ToAPI: Sized {
    /// Generate a JSON representation of the model for public API use.
    fn api(&self) -> impl std::future::Future<Output = serde_json::Value>;
}

/// Converts a slice of items implementing the `ToAPI` trait into a `Vec` of JSON values.
///
/// This function is asynchronous and iterates over a slice of items, calling the `api`
/// method on each to generate its JSON representation. The results are collected into
/// a `Vec<serde_json::Value>`.
///
/// # Arguments
/// * `items` - A slice of items implementing the `ToAPI` trait.
///
/// # Returns
/// An asynchronous computation that resolves to a `Vec` of JSON values.
pub async fn vec_to_api(items: &[impl ToAPI]) -> Vec<serde_json::Value> {
    let mut ret = Vec::with_capacity(items.len());

    for e in items {
        ret.push(e.api().await);
    }

    ret
}

/// Converts a string into a `Uuid`.
///
/// This function attempts to parse a string as a UUID. If the parsing fails,
/// it returns an `ApiError` indicating the failure.
///
/// # Arguments
/// * `id` - A string slice representing the UUID to be parsed.
///
/// # Returns
/// * `Ok(uuid::Uuid)` if the string is a valid UUID.
/// * `Err(ApiError)` if the string is not a valid UUID, containing a descriptive error message.
pub fn to_uuid(id: &str) -> Result<uuid::Uuid, ApiError> {
    uuid::Uuid::from_str(id).map_err(|_| no_uuid_error())
}

/// Generates an `ApiError` indicating that the input is not a valid UUID.
///
/// This function is used internally to standardize error responses for invalid UUIDs.
///
/// # Returns
/// * `ApiError` - A `BadRequest` error with a JSON payload describing the issue.
#[must_use]
pub fn no_uuid_error() -> ApiError {
    api_error("No valid UUID")
}

/// Generates a custom `ApiError` with a given message.
///
/// This function creates a `BadRequest` response with a JSON payload containing
/// an error message, making it useful for consistent error reporting.
///
/// # Arguments
/// * `msg` - A string slice containing the error message to be included in the response.
///
/// # Returns
/// * `ApiError` - A `BadRequest` error with a JSON payload describing the issue.
#[must_use]
pub fn api_error(msg: &str) -> ApiError {
    BadRequest(json!({
        "error": msg
    }))
}
