use rocket::http::{ContentType, Status};

#[cfg(feature = "cache")]
pub mod cache;

pub mod api;
pub mod assets;
mod context;

pub use context::RequestContext;

/// HTTP response containing a string payload.
pub type StringResponse = (Status, (ContentType, String));

pub trait RespondString {
    fn respond(self) -> StringResponse;
}

impl RespondString for String {
    fn respond(self) -> StringResponse {
        (Status::Ok, (ContentType::Text, self))
    }
}

/// HTTP response containing raw binary data.
pub type RawResponse = (Status, (ContentType, Vec<u8>));

pub trait RespondRaw {
    fn respond(self) -> RawResponse;
}

impl RespondRaw for Vec<u8> {
    fn respond(self) -> RawResponse {
        (Status::Ok, (ContentType::Binary, self))
    }
}

impl RespondRaw for StringResponse {
    fn respond(self) -> RawResponse {
        (self.0, (self.1.0, self.1.1.into_bytes()))
    }
}

/// Helper function to create a JSON HTTP response.
///
/// # Parameters
/// - `json`: A reference to a `serde_json::Value` representing the JSON payload.
///
/// # Returns
/// A `StringResponse` with status `200 OK`, content type `application/json`, and the JSON-encoded body.
#[must_use]
pub fn respond_json(json: &serde_json::Value) -> StringResponse {
    (
        Status::Ok,
        (
            ContentType::JSON,
            serde_json::to_string(json).expect("Failed to serialize JSON"),
        ),
    )
}

/// Helper function to create an HTML HTTP response.
///
/// # Parameters
/// - `html`: A string slice (`&str`) containing the HTML content.
///
/// # Returns
/// A `StringResponse` with status `200 OK`, content type `text/html`, and the HTML content as the body.
#[must_use]
pub fn respond_html(html: String) -> StringResponse {
    (Status::Ok, (ContentType::HTML, html))
}

/// Helper function to create an JS HTTP response.
///
/// # Parameters
/// - `html`: A string slice (`&str`) containing the JS content.
///
/// # Returns
/// A `StringResponse` with status `200 OK`, content type `text/javascript`, and the JS content as the body.
#[must_use]
pub fn respond_script(script: String) -> StringResponse {
    (Status::Ok, (ContentType::JavaScript, script))
}

/// Creates a custom HTTP response with the specified status, content type, and body.
///
/// # Parameters
/// - `status`: The HTTP status code to include in the response.
/// - `content_type`: The content type of the response body (e.g., `ContentType::JSON` or `ContentType::HTML`).
/// - `body`: The response body as a `Vec<u8>`.
///
/// # Returns
/// A `RawResponse` containing the provided status, content type, and body.
#[must_use]
pub const fn respond_with(status: Status, content_type: ContentType, body: Vec<u8>) -> RawResponse {
    (status, (content_type, body))
}
